<?php

namespace Drupal\console_extras\Command;

use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Drupal\Console\Annotations\DrupalCommand;
use Drupal\Console\Core\Command\Command;
use Drupal\Console\Core\Command\ContainerAwareCommand;
use Drupal\Console\Utils\Validator;
use Drupal\Console\Extension\Manager;
use Drupal\Console\Core\Utils\StringConverter;
use Drupal\Console\Core\Utils\ChainQueue;
use Drupal\Console\Command\Shared\ModuleTrait;
use Drupal\console_extras\Generator\BatchGenerator;


/**
 * Class BatchCommand.
 *
 * @DrupalCommand (
 *     extension="console_extras",
 *     extensionType="module"
 * )
 */
class BatchCommand extends ContainerAwareCommand {

  use ModuleTrait;

  /**
   * @var BatchGenerator
   */
  protected $generator;

  /**
   * @var Manager
   */
  protected $extensionManager;

  /**
   * @var Validator
   */
  protected $validator;

  /**
   * @var StringConverter
   */
  protected $stringConverter;

  /**
   * @var ChainQueue
   */
  protected $chainQueue;

  /**
   * @var string
   */
  protected $appRoot;

  /**
   * BatchCommand constructor.
   * @param BatchGenerator $generator
   * @param Manager $extensionManager
   * @param Validator $validator
   * @param StringConverter $stringConverter
   * @param ChainQueue $chainQueue
   * @param $appRoot
   */
  public function __construct(
    BatchGenerator $generator,
    Manager $extensionManager,
    Validator $validator,
    StringConverter $stringConverter,
    ChainQueue $chainQueue,
    $appRoot
  ) {
    $this->generator = $generator;
    $this->extensionManager = $extensionManager;
    $this->validator = $validator;
    $this->stringConverter = $stringConverter;
    $this->chainQueue = $chainQueue;
    $this->appRoot = $appRoot;
    parent::__construct();
  }

  /**
   * {@inheritdoc}
   */
  protected function configure() {
    $this
      ->setName('generate:extra:batch')
      ->setDescription($this->trans('commands.generate.extra.batch.description'))
      ->addOption(
        'module',
        null,
        InputOption::VALUE_REQUIRED,
        $this->trans('commands.common.options.module')
      )
      ->addOption(
        'form_class',
        null,
        InputOption::VALUE_REQUIRED,
        $this->trans('commands.generate.form.options.class')
      )
    ;
  }

  /**
   * {@inheritdoc}
   */
  protected function initialize(InputInterface $input, OutputInterface $output) {
    parent::initialize($input, $output);
  }

  /**
   * {@inheritdoc}
   */
  protected function interact(InputInterface $input, OutputInterface $output) {
    // --module option
    $module = $this->getModuleOption();

    // --form_class option
    $className = $input->getOption('form_class');
    if (!$className) {
      $className = $this->getIo()->ask(
        $this->trans('commands.generate.form.questions.class'),
        'DefaultBatchForm',
        function ($className) use ($module) {
          // Gets the destination module's path.
          $modulePath = drupal_get_path('module', $module);

          // Checks if the class files exist.
          $fullPath = $modulePath . '/src/Form/' . $className . '.php';
          if (file_exists($fullPath)) {
            throw new \InvalidArgumentException(
              sprintf(
                $this->trans('commands.generate.module.errors.directory-exists'),
                $fullPath
              )
            );
          }
          $fullPath = $modulePath . '/src/Entity/' . $className . '.php';
          if (file_exists($fullPath)) {
            throw new \InvalidArgumentException(
              sprintf(
                $this->trans('commands.generate.module.errors.directory-exists'),
                $fullPath
              )
            );
          }

          return $this->validator->validateClassName($className);
        }
      );
      $input->setOption('form_class', $className);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function execute(InputInterface $input, OutputInterface $output) {
    $module = $input->getOption('module');
    $form_class = $input->getOption('form_class');
    $form_class_underscore = $this->stringConverter->camelCaseToMachineName($form_class);

    $this->generator->generate(['module' => $module, 'form_class' => $form_class]);
    $this->getIo()->info($this->trans('commands.generate.extra.batch.messages.success'));
    $this->getIo()->info('You can now visit the batch page at /admin/content/batch/' . $form_class_underscore . '.');

    $this->chainQueue->addCommand('cache:rebuild', ['cache' => 'all']);
  }

}
